/* This is my (Ed Mandy) PVoice library.   v.0999

   I used GBDK v 2.1.5 to compile the object file, but I have also included
   the .s file so you can edit it (unless this is a binary-only release).
   If you do edit it, please tell me so I can add the new functionality to
   the library.

   Oh, and all I ask is that you give me credit if you use this in any
   of your creations.  If you do use it, I'd really like to hear about it,
   but in any case, please just give me credit in your program.

   Thanks,
   Ed Mandy
   emandy@atcorp.com
   ICQ:  17868116
*/

/*  A macro that will load bank b into the switchable ROM bank of the
    PocketVoice.
    Note:  On the PocketVoice, the first 24k is fixed, and the last 8k
    is switchable.
*/
#define SWITCH_ROM_PV(b) \
  *(unsigned char *)0x7000 = ((UWORD) b)>>8; \
  *(unsigned char *)0x6000 = ((UWORD) b)&0xFF

/*  Call this before recording/playing.
    */
void initPVoice();


/*  playVoice will play numBytes of data from the buffer that sound points
    to.  It will return the number of bytes that were played.
*/
UWORD playVoice(const void *sound, UWORD numBytes);

/*  recVoice will record numBytes of data to the buffer that sound points
    to.  It will return the number of bytes that were recorded.

    If you release the A button, recVoice will stop recording.
    I would recommend asking the user to press-and-hold the A button
    until they are done recording.  Call recVoice once they have pressed
    the A button, and recVoice will return either when the A button has
    been released or when numBytes have been recorded (whichever comes
    first).
*/
UWORD recVoice(const void *sound, UWORD numBytes);

/*   PVROMcpy
     This is supposed to copy to (or from) PocketVoice ROM.

     Here's what Bung says about it.
		;; If you want to write data to Pocket Voice's Flash Rom, it needs to
		;; copy some routines includes the writing Flash Rom main routine to
		;; GB's work Ram 0xc000--0xdfff and jump to run it. It is because during
		;; flashing, the whole Flash Rom can't be read as normal, some times it
		;; just be read and checked in it's d7 or d6 only for waiting last page
		;; written ready. So some routines need to be run in GB's work ram area.

     returns numBytes

     PVROMcpy will call flashROM.  flashRom can only copy 0x100 bytes from src
     to dest.
     src must be in work ram (C000-DFFF)
         the 0x0100 bytes should be 0xXX00-0xXXFF
     dest must be in 6000-7FFF (switchable bank)
     bank tells what bank to load to 6000-7FFF
     It will copy some code to RAM and then execute it
     *** NOTE *** the code is copied to 0xD000, so do not place src in this area

     NOTE:  numBytes is the number of bytes / it better be <= 0x100

     use PVROMcpy, as it should be much safer/easier to use
*/
UWORD flashROM(void *dest, const void *src, UBYTE bank);
UWORD PVROMcpy(void *dest, const void *src, UWORD numBytes, UWORD bank)
{
	//this will copy src to dest in 0x70 byte chunks
   //  it will copy 0x70 bytes from src to 0xDE00
   //  then, it will call flashROM wits 0xDF00 as src

   //UWORD numBytesCopied = 0, numToCopy = 0, tempWord;
   char *RAM_temp = (char *)0xDF00;

   if(numBytes == 0)
   	return 0;	//waste of time
   
   memcpy(RAM_temp, src, numBytes);
   wait_VBL_done();
   flashROM(dest, RAM_temp, bank);//copy 0x0100 bytes

   //SWITCH_ROM_PV(3);//something gets screwed up, so that these writes lock it

   return 0;  //numBytesCopied;
}


/*  setPVoffset will set the offset for the PocketVoice.
    It copies the offset-switching code to RAM and then executes it.
    At the end of the execution is code that resets the GB, so it will
    start whatever was at that offset.

    The second parameter is the CPU type.  It's usually just "_cpu" in GBDK.

    You will not return from this function!!!

    Example:  If you had a ROM that started at the second 32k of the
    PocketVoice, you could load/run that ROM by calling setPVoffset(1).

    Note:  Calling setPVoffset(0) will (effectively) do the same as turning
    off your GameBoy and turning it back on.
*/
void _setPVoffset(char offset, char _cpu);
void setPVoffset(char offset)
{
   _setPVoffset(offset, _cpu);
}


